#include "resampler.h"
using namespace Xybrid::NodeLib;


#include <iostream>
#include <array>

#ifdef WITH_BOOST
#include <boost/math/special_functions/bessel.hpp>
#define cyl_bessel_i boost::math::cyl_bessel_i
#else
#include <cmath>
#define cyl_bessel_i std::cyl_bessel_i
#endif

namespace {
    const constexpr double PI = 3.141592653589793238462643383279502884197169399375105820974;

    const constexpr double KAISER_ALPHA = 7.5;
    const constexpr double KAISER_BETA = PI * KAISER_ALPHA;

    inline constexpr double sinc(double x) {
        if (x == 0) return 1;
        double px = x * PI;
        return std::sin(px) / px;
    }
}



// generate
const std::array<std::array<double, LUT_TAPS>, LUT_STEPS> Xybrid::NodeLib::resamplerLUT = [] {

    double denom = cyl_bessel_i(0, KAISER_BETA);

    std::array<std::array<double, LUT_TAPS>, LUT_STEPS> t;
    t[0] = {0, 0, 0, 1, 0, 0, 0, 0}; // we already know the ideal integer step
    for (size_t step = 1; step < LUT_STEPS; step++) {
        double sv = static_cast<double>(step) / LUT_STEPS;
        for (size_t tap = 0; tap < LUT_TAPS; tap++) {
            double x = static_cast<double>(tap) - sv;
              t[step][tap] = sinc(x-(LUT_TAPS/2-1)) * (cyl_bessel_i(0, KAISER_BETA * std::sqrt(1 - std::pow(((2 * (x+1)) / (LUT_TAPS)) - 1, 2))) / denom);
            if (t[step][tap] != t[step][tap]) t[step][tap] = 0; // NaN guard
            //std::cout << "tap " << tap << ": " << t[step][tap] << " ";
        }
        //std::cout << "\n";
    }
    return t;
}();
